@@ -8,7 +8,7 @@ import numpy
 
 from safe.common.interpolation2d import interpolate_raster
 from safe.common.utilities import verify
-from safe.common.utilities import ugettext as _
+from safe.common.utilities import ugettext as tr
 from safe.common.numerics import ensure_numeric
 from safe.common.geodesy import Point
 from safe.common.exceptions import InaSAFEError, BoundsError
@@ -19,6 +19,7 @@ from safe.common.polygon import (inside_polygon,
 from safe.storage.vector import Vector, convert_polygons_to_centroids
 from safe.storage.utilities import geometrytype2string
 from safe.storage.utilities import DEFAULT_ATTRIBUTE
+from safe.storage.geometry import Polygon
 
 
 def assign_hazard_values_to_exposure_data(hazard, exposure,
@@ -39,8 +40,7 @@ def assign_hazard_values_to_exposure_data(hazard, exposure,
        * attribute_name:
              If hazard layer is of type raster, this is the name for new
              attribute in the result containing the hazard level.
-             If None (default) the name of hazard is used
-
+             If None (default) the name of hazard is used.
              If hazard layer is of type vector, it is the name of the
              attribute to transfer from the hazard layer into the result.
              If None (default) all attributes are transferred.
@@ -91,6 +91,16 @@ def assign_hazard_values_to_exposure_data(hazard, exposure,
         Raster-Raster:   Raster data
     """
 
+    # Make sure attribute name can be stored in a shapefile
+    if attribute_name is not None and len(attribute_name) > 10:
+        msg = ('Specified attribute name "%s"\
+         has length = %i. '
+               'To fit into a shapefile it must be at most 10 characters '
+               'long. How about naming it "%s"?' % (attribute_name,
+                                                    len(attribute_name),
+                                                    attribute_name[:10]))
+        raise InaSAFEError(msg)
+
     layer_name, attribute_name = check_inputs(hazard, exposure,
                                               layer_name, attribute_name)
     # Raster-Vector
@@ -265,7 +275,7 @@ def interpolate_polygon_vector(source, target,
         # that of the original polygon
         R = Vector(data=P.get_data(),
                    projection=P.get_projection(),
-                   geometry=X.get_geometry(),
+                   geometry=target.get_geometry(as_geometry_objects=True),
                    name=P.get_name())
     else:
         msg = ('Unknown datatype for polygon2vector interpolation: '
@@ -303,7 +313,13 @@ def interpolate_polygon_raster(source, target,
     verify(source.is_polygon_data)
 
     # Run underlying clipping algorithm
-    polygon_geometry = source.get_geometry()
+    polygon_geometry = source.get_geometry(as_geometry_objects=True)
+
+    # FIXME (Ole): Perhaps refactor so that polygon_geometry can
+    # be passed in directly. See issue #324, comment
+    #https://github.com/AIFDR/inasafe/issues/324#issuecomment-9440584
+    # FIXME (Ole): Not sure if needed, but explicitly make sure
+    # exposure raster is *never* scaled here
     polygon_attributes = source.get_data()
     res = clip_grid_by_polygons(target.get_data(),
                                 target.get_geotransform(),
@@ -385,12 +401,14 @@ def interpolate_raster_vector_points(source, target,
     N = len(target)
     if attribute_name is None:
         attribute_name = source.get_name()
+        # FIXME (Ole): Launder for shape files
+        attribute_name = str(attribute_name[:10])
 
     try:
         values = interpolate_raster(longitudes, latitudes, A,
                                     coordinates, mode=mode)
     except (BoundsError, InaSAFEError), e:
-        msg = (_('Could not interpolate from raster layer %(raster)s to '
+        msg = (tr('Could not interpolate from raster layer %(raster)s to '
                  'vector layer %(vector)s. Error message: %(error)s')
                % {'raster': source.get_name(),
                   'vector': target.get_name(),
@@ -456,10 +474,14 @@ def interpolate_polygon_points(source, target,
     original_geometry = target.get_geometry()  # Geometry for returned data
 
     # Extract polygon features
-    geom = source.get_geometry()
+    geom = source.get_geometry(as_geometry_objects=True)
     data = source.get_data()
     verify(len(geom) == len(data))
 
+    # Include polygon_id as attribute
+    attribute_names.append('polygon_id')
+    attribute_names.append(DEFAULT_ATTRIBUTE)
+
     # Augment point features with empty attributes from polygon
     for a in attributes:
         if attribute_name is None:
@@ -469,10 +491,12 @@ def interpolate_polygon_points(source, target,
         else:
             # Use only requested attribute
             # FIXME (Ole): Test for this is not finished
+            # BUT don't do it....Issue #251
             a[attribute_name] = None
 
         # Always create default attribute flagging if point was
         # inside any of the polygons
+        # FIXME (Ole): Remove as already appended.....
         a[DEFAULT_ATTRIBUTE] = None
 
     # Traverse polygons and assign attributes to points that fall inside
@@ -488,11 +512,14 @@ def interpolate_polygon_points(source, target,
         poly_attr[DEFAULT_ATTRIBUTE] = True
 
         # Clip data points by polygons and add polygon attributes
-        indices = inside_polygon(points, polygon)
+        indices = inside_polygon(points, polygon.outer_ring,
+                                 holes=polygon.inner_rings)
+
         for k in indices:
             for key in poly_attr:
                 # Assign attributes from polygon to points
                 attributes[k][key] = poly_attr[key]
+            attributes[k]['polygon_id'] = i  # Store id for associated polygon
 
     # Create new Vector instance and return
     V = Vector(data=attributes,
@@ -594,21 +621,50 @@ def interpolate_raster_raster(source, target):
 
 
 # FIXME (Ole): Not sure this is the place for this function
-def make_circular_polygon(center, radius):
-    """Create circula polygon in geographic coordinates
+def make_circular_polygon(centers, radii, attributes=None):
+    """Create circular polygon in geographic coordinates
 
     Args:
-        center: list of longitude, latitude
-        radius: desired approximate radius in meters
+        centers: list of (longitude, latitude)
+        radii: desired approximate radii in meters (must be
+               monotonically ascending).
+        Can be either one number or list of numbers
+        attributes (optional): Attributes for each center
     Returns
         Vector polygon layer representing circle in WGS84
     """
 
-    # FIXME (Ole): Input checks here
-    p = Point(longitude=center[0], latitude=center[1])
-    C = p.generate_circle(radius)
-    Z = Vector(geometry=[C],  # List with one element containing the polygon
-               data=[{'Radius': radius}],
+    if not isinstance(radii, list):
+        radii = [radii]
+
+    # FIXME (Ole): Check that radii are monotonically increasing
+
+    circles = []
+    new_attributes = []
+    for i, center in enumerate(centers):
+        p = Point(longitude=center[0], latitude=center[1])
+        inner_rings = None
+        for radius in radii:
+            # Generate circle polygon
+            C = p.generate_circle(radius)
+            circles.append(Polygon(outer_ring=C, inner_rings=inner_rings))
+
+            # Store current circle and inner ring for next poly
+            inner_rings = [C]
+
+            # Carry attributes for center forward
+            attr = {}
+            if attributes is not None:
+                for key in attributes[i]:
+                    attr[key] = attributes[i][key]
+
+            # Add radius to this ring
+            attr['Radius'] = radius
+
+            new_attributes.append(attr)
+
+    Z = Vector(geometry=circles,  # List with circular polygons
+               data=new_attributes,  # Associated attributes
                geometry_type='polygon')
 
     return Z
